@@ -1,3 +1,5 @@
+from __future__ import with_statement
+from contextlib import contextmanager
 from datetime import datetime
 from UserDict import DictMixin
 
@@ -6,26 +8,35 @@ from sqlalchemy import Column
 from sqlalchemy import Boolean
 from sqlalchemy import Integer
 from sqlalchemy import DateTime
-from sqlalchemy import Table
 from sqlalchemy import Unicode
 from sqlalchemy import func
 from sqlalchemy.sql.expression import or_
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm.exc import NoResultFound
 from pyramid.location import lineage
-from pyramid.security import Allow
-from pyramid.security import ALL_PERMISSIONS
-from pyramid.security import view_execution_permitted
 from pyramid.security import authenticated_userid
+from pyramid.security import has_permission as base_has_permission
+from pyramid.security import view_execution_permitted
 
 from kotti import get_settings
 from kotti import DBSession
-from kotti import metadata
+from kotti import Base
+from kotti.util import _
 from kotti.util import JsonType
 from kotti.util import request_cache
 from kotti.util import DontCache
 
-class Principal(object):
+def get_principals():
+    return get_settings()['kotti.principals_factory'][0]()
+
+def get_user(request):
+    userid = authenticated_userid(request)
+    return get_principals().get(userid)
+
+def has_permission(permission, context, request):
+    with authz_context(context, request):
+        return base_has_permission(permission, context, request)
+
+class Principal(Base):
     """A minimal 'Principal' implementation.
 
     The attributes on this object correspond to what one ought to
@@ -44,6 +55,22 @@ class Principal(object):
         receiver of the email.  This attribute should be set to
         'None' once confirmation has succeeded.
     """
+    __tablename__ = 'principals'
+    __mapper_args__ = dict(
+        order_by='principals.name',
+        )
+
+    id = Column(Integer, primary_key=True)
+    name = Column(Unicode(100), unique=True)
+    password = Column(Unicode(100))
+    active = Column(Boolean)
+    confirm_token = Column(Unicode(100))
+    title = Column(Unicode(100), nullable=False)
+    email = Column(Unicode(100), unique=True)
+    groups = Column(JsonType(), nullable=False)
+    creation_date = Column(DateTime(), nullable=False)
+    last_login_date = Column(DateTime())
+
     def __init__(self, name, password=None, active=True, confirm_token=None,
                  title=u"", email=None, groups=()):
         self.name = name
@@ -129,15 +156,18 @@ class AbstractPrincipals(object):
         """
 
 ROLES = {
-    u'role:viewer': Principal(u'role:viewer', title=u'Viewer'),
-    u'role:editor': Principal(u'role:editor', title=u'Editor'),
-    u'role:owner': Principal(u'role:owner', title=u'Owner'),
-    u'role:admin': Principal(u'role:admin', title=u'Admin'),
+    u'role:viewer': Principal(u'role:viewer', title=_(u'Viewer')),
+    u'role:editor': Principal(u'role:editor', title=_(u'Editor')),
+    u'role:owner': Principal(u'role:owner', title=_(u'Owner')),
+    u'role:admin': Principal(u'role:admin', title=_(u'Admin')),
     }
+_DEFAULT_ROLES = ROLES.copy()
 
 # These roles are visible in the sharing tab
 SHARING_ROLES = [u'role:viewer', u'role:editor', u'role:owner']
 USER_MANAGEMENT_ROLES = SHARING_ROLES + ['role:admin']
+_DEFAULT_SHARING_ROLES = SHARING_ROLES[:]
+_DEFAULT_USER_MANAGEMENT_ROLES = USER_MANAGEMENT_ROLES[:]
 
 # This is the ACL that gets set on the site root on creation.
 SITE_ACL = [
@@ -147,50 +177,50 @@ SITE_ACL = [
     ['Allow', 'role:owner', ['view', 'add', 'edit', 'manage']],
     ]
 
+def set_roles(roles_dict):
+    ROLES.clear()
+    ROLES.update(roles_dict)
+
+def set_sharing_roles(role_names):
+    SHARING_ROLES[:] = role_names
+
+def set_user_management_roles(role_names):
+    USER_MANAGEMENT_ROLES[:] = role_names
+
+def reset_roles():
+    ROLES.clear()
+    ROLES.update(_DEFAULT_ROLES)
+
+def reset_sharing_roles():
+    SHARING_ROLES[:] = _DEFAULT_SHARING_ROLES
+
+def reset_user_management_roles():
+    USER_MANAGEMENT_ROLES[:] = _DEFAULT_USER_MANAGEMENT_ROLES
+
+def reset():
+    reset_roles()
+    reset_sharing_roles()
+    reset_user_management_roles()
+
 class PersistentACLMixin(object):
-    """Manages access to ``self._acl`` which is a JSON- serialized
-    representation of ``self.__acl__``.
-    """
-    ALL_PERMISSIONS_SERIALIZED = '__ALL_PERMISSIONS__'
-
-    @staticmethod
-    def _deserialize_ace(ace):
-        ace = list(ace)
-        if ace[2] == PersistentACLMixin.ALL_PERMISSIONS_SERIALIZED:
-            ace[2] = ALL_PERMISSIONS
-        return tuple(ace)
-
-    @staticmethod
-    def _serialize_ace(ace):
-        ace = list(ace)
-        if ace[2] == ALL_PERMISSIONS:
-            ace[2] = PersistentACLMixin.ALL_PERMISSIONS_SERIALIZED
-        return ace
-
     def _get_acl(self):
-        if self._acl is not None:
-            acl = self._default_acl() + self._acl
-            return [self._deserialize_ace(ace) for ace in acl]
-        else:
+        if self._acl is None:
             raise AttributeError('__acl__')
+        return self._acl
 
-    def _set_acl(self, acl):
-        self._acl = [self._serialize_ace(ace) for ace in acl]
+    def _set_acl(self, value):
+        self._acl = value
 
     def _del_acl(self):
-        if self._acl is not None:
-            self._acl = None
-        else:
-            raise AttributeError('__acl__')
+        self._acl = None
 
     __acl__ = property(_get_acl, _set_acl, _del_acl)
 
-    def _default_acl(self):
-        # ACEs that will be put on top, no matter what
-        return [
-            (Allow, 'role:admin', ALL_PERMISSIONS),
-            ]
+def _cachekey_list_groups_raw(name, context):
+    context_id = context is not None and getattr(context, 'id', id(context))
+    return (name, context_id)
 
+@request_cache(_cachekey_list_groups_raw)
 def list_groups_raw(name, context):
     """A set of group names in given ``context`` for ``name``.
 
@@ -220,7 +250,7 @@ def _cachekey_list_groups_ext(name, context=None, _seen=None, _inherited=None):
     if _seen is not None or _inherited is not None:
         raise DontCache
     else:
-        context_id = context is not None and context.id
+        context_id = context is not None and getattr(context, 'id', id(context))
         return (name, context_id)
 
 @request_cache(_cachekey_list_groups_ext)
@@ -286,12 +316,20 @@ def list_groups_callback(name, request):
             context = get_root(request)
         return list_groups(name, context)
 
-def view_permitted(context, request, name=''):
+@contextmanager
+def authz_context(context, request):
+    before = request.environ.pop('authz_context', None)
+    request.environ['authz_context'] = context
     try:
-        request.environ['authz_context'] = context
-        return view_execution_permitted(context, request, name)
+        yield
     finally:
         del request.environ['authz_context']
+        if before is not None:
+            request.environ['authz_context'] = before
+
+def view_permitted(context, request, name=''):
+    with authz_context(context, request):
+        return view_execution_permitted(context, request, name)
 
 def principals_with_local_roles(context, inherit=True):
     """Return a list of principal names that have local roles in the
@@ -326,18 +364,11 @@ def map_principals_with_local_roles(context):
             value.append((principal, (all, inherited)))
     return sorted(value, key=lambda t: t[0].name)
 
-def get_principals():
-    return get_settings()['kotti.principals_factory'][0]()
-
 def is_user(principal):
     if not isinstance(principal, basestring):
         principal = principal.name
     return ':' not in principal
 
-def get_user(request):
-    userid = authenticated_userid(request)
-    return get_principals().get(userid)
-
 class Principals(DictMixin):
     """Kotti's default principal database.
 
@@ -415,18 +446,3 @@ class Principals(DictMixin):
 
 def principals_factory():
     return Principals()
-
-principals_table = Table('principals', metadata,
-    Column('id', Integer, primary_key=True),
-    Column('name', Unicode(100), unique=True),
-    Column('password', Unicode(100)),
-    Column('active', Boolean),
-    Column('confirm_token', Unicode(100)),
-    Column('title', Unicode(100), nullable=False),
-    Column('email', Unicode(100), unique=True),
-    Column('groups', JsonType(), nullable=False),
-    Column('creation_date', DateTime(), nullable=False),
-    Column('last_login_date', DateTime()),
-)
-
-mapper(Principal, principals_table, order_by=principals_table.c.name)
