@@ -1,3 +1,4 @@
+from collections import defaultdict
 from datetime import datetime
 import hashlib
 import urllib
@@ -5,42 +6,87 @@ import urllib
 from babel.dates import format_date
 from babel.dates import format_datetime
 from babel.dates import format_time
+import colander
+import deform
+from deform import Button
 from pyramid.decorator import reify
 from pyramid.httpexceptions import HTTPFound
+from pyramid.i18n import get_localizer
 from pyramid.i18n import get_locale_name
+from pyramid.i18n import make_localizer
+from pyramid.interfaces import ITranslationDirectories
 from pyramid.location import inside
 from pyramid.location import lineage
 from pyramid.renderers import get_renderer
 from pyramid.renderers import render
-from pyramid.security import has_permission
-from pyramid.url import resource_url
+from pyramid.threadlocal import get_current_registry
+from pyramid.threadlocal import get_current_request
 from pyramid.view import render_view_to_response
-from deform import ValidationFailure
+from pyramid_deform import FormView
+from pyramid_deform import CSRFSchema
 
 from kotti import get_settings
 from kotti import DBSession
+from kotti.util import _
 from kotti.util import title_to_name
 from kotti.events import objectevent_listeners
-from kotti.resources import Node
-from kotti.security import view_permitted
+from kotti.resources import Content
 from kotti.security import get_user
+from kotti.security import has_permission
+from kotti.security import view_permitted
 from kotti.views.slots import slot_events
 
+
 def template_api(context, request, **kwargs):
     return get_settings()['kotti.templates.api'][0](
         context, request, **kwargs)
 
+
 def render_view(context, request, name='', secure=True):
     response = render_view_to_response(context, request, name, secure)
-    return response.ubody
+    if response is not None:
+        return response.ubody
+
 
 def add_renderer_globals(event):
     if event['renderer_name'] != 'json':
-        api = getattr(event['request'], 'template_api', None)
-        if api is None:
+        request = event['request']
+        api = getattr(request, 'template_api', None)
+        if api is None and request is not None:
             api = template_api(event['context'], event['request'])
         event['api'] = api
 
+
+def is_root(context, request):
+    return context is TemplateAPI(context, request).root
+
+
+def get_localizer_for_locale_name(locale_name):
+    registry = get_current_registry()
+    tdirs = registry.queryUtility(ITranslationDirectories, default=[])
+    return make_localizer(locale_name, tdirs)
+
+
+def translate(*args, **kwargs):
+    request = get_current_request()
+    if request is None:
+        localizer = get_localizer_for_locale_name('en')
+    else:
+        localizer = get_localizer(request)
+    return localizer.translate(*args, **kwargs)
+
+
+class TemplateStructure(object):
+    def __init__(self, html):
+        self.html = html
+
+    def __html__(self):
+        return self.html
+    __unicode__ = __html__
+
+    def __getattr__(self, key):
+        return getattr(self.html, key)
+
 class Slots(object):
     def __init__(self, context, request):
         self.context = context
@@ -77,6 +123,8 @@ class TemplateAPI(object):
     EDIT_MASTER = 'kotti:templates/edit/master.pt'
     SITE_SETUP_MASTER = 'kotti:templates/site-setup/master.pt'
 
+    body_css_class = ''
+
     def __init__(self, context, request, bare=None, **kwargs):
         self.context, self.request = context, request
 
@@ -98,7 +146,10 @@ class TemplateAPI(object):
 
     @reify
     def site_title(self):
-        return get_settings().get('kotti.site_title', self.root.title)
+        value = get_settings().get('kotti.site_title')
+        if not value:
+            value = self.root.title
+        return value
 
     @reify
     def page_title(self):
@@ -108,15 +159,10 @@ class TemplateAPI(object):
         view_title += self.context.title
         return u'%s - %s' % (view_title, self.site_title)
 
-    @reify
-    def first_heading(self):
-        return u'<h1>%s</h1>' % self.page_title
-
-    def url(self, context=None, *elements):
+    def url(self, context=None, *elements, **kwargs):
         if context is None:
             context = self.context
-        rhs = '/'.join(elements)
-        return resource_url(context, self.request) + rhs
+        return self.request.resource_url(context, *elements, **kwargs)
 
     @reify
     def root(self):
@@ -126,6 +172,10 @@ class TemplateAPI(object):
     def lineage(self):
         return list(lineage(self.context))
 
+    @reify
+    def breadcrumbs(self):
+        return reversed(self.lineage)
+
     @reify
     def user(self):
         return get_user(self.request)
@@ -140,34 +190,25 @@ class TemplateAPI(object):
             context = self.context
         if request is None:
             request = self.request
-        return render_view(context, request, name, secure)
+        return TemplateStructure(render_view(context, request, name, secure))
 
     def render_template(self, renderer, **kwargs):
-        return render(renderer, kwargs, self.request)
+        return TemplateStructure(render(renderer, kwargs, self.request))
 
     def list_children(self, context=None, permission='view'):
         if context is None:
             context = self.context
         children = []
-        for child in context.values():
-            if (not permission or
-                has_permission(permission, child, self.request)):
-                children.append(child)
+        if hasattr(context, 'values'):
+            for child in context.values():
+                if (not permission or
+                    has_permission(permission, child, self.request)):
+                    children.append(child)
         return children
 
-    def list_children_go_up(self, context=None, permission='view'):
-        if context is None:
-            context = self.context
-        parent = context
-        children = self.list_children(context, permission)
-        if not children and context.__parent__ is not None:
-            parent = context.__parent__
-            children = self.list_children(parent)
-        return (parent, children)
-
     inside = staticmethod(inside)
 
-    def avatar_url(self, user=None, size="20", default_image='identicon'):
+    def avatar_url(self, user=None, size="14", default_image='identicon'):
         if user is None:
             user = self.user
         email = user.email
@@ -205,7 +246,7 @@ class TemplateAPI(object):
             if class_.type_info.name == name:
                 return class_
 
-    def _find_edit_view(self, item):
+    def find_edit_view(self, item):
         view_name = self.request.view_name
         if not view_permitted(item, self.request, view_name):
             view_name = u'edit'
@@ -213,37 +254,10 @@ class TemplateAPI(object):
             view_name = u''
         return view_name
 
-    def _make_links(self, items):
-        links = []
-        for item in items:
-            view_name = self._find_edit_view(item)
-            view_name = view_name and '@@' + view_name
-            url = resource_url(item, self.request) + view_name
-            links.append(dict(
-                url=url,
-                name=item.title,
-                is_edit_link=view_name != '',
-                node=item,
-                is_context=item == self.context,
-                ))
-        return links
-
-    @reify
-    def breadcrumbs(self):
-        return self._make_links(tuple(reversed(self.lineage)))
-
-    @reify
-    def context_links(self):
-        siblings = []
-        if self.context.__parent__ is not None:
-            siblings = self._make_links(
-                self.list_children(self.context.__parent__))
-            siblings = filter(lambda l: not l['is_context'], siblings)
-        children = self._make_links(self.list_children(self.context))
-        return siblings, children
-
     @reify
     def edit_links(self):
+        if not hasattr(self.context, 'type_info'):
+            return []
         return [l for l in self.context.type_info.edit_links
                 if l.permitted(self.context, self.request)]
 
@@ -251,55 +265,6 @@ class TemplateAPI(object):
         return [l for l in getattr(self, name)
                 if l.permitted(self.context, self.request)]
 
-    def get_paste_item(self):
-        info = self.request.session.get('kotti.paste')
-        if info:
-            id, action = info
-            item = DBSession().query(Node).get(id)
-            if not item.type_info.addable(self.context, self.request):
-                return
-            if action == 'cut' and self.inside(self.context, item):
-                return
-            if self.context == item:
-                return
-            return item
-
-def addable_types(context, request):
-    all_types = get_settings()['kotti.available_types']
-
-    # 'possible_parents' is a list of dicts with 'node' and 'factories',
-    # where 'node' is the context to add to and 'factories' is the list
-    # of factories that can be applied in the contetx:
-    possible_parents = []
-    parent = context
-    while parent is not None:
-        possible_parents.append({'node': parent, 'factories': []})
-        parent = parent.__parent__
-
-    for entry in possible_parents:
-        parent = entry['node']
-        for factory in all_types:
-            if factory.type_info.addable(parent, request):
-                entry['factories'].append(factory)
-
-    possible_parents = filter(lambda e: e['factories'], possible_parents)
-
-    _possible_types = {}
-    for entry in possible_parents:
-        for factory in entry['factories']:
-            name = factory.type_info.name
-            pt = _possible_types.get(
-                name, {'factory': factory, 'nodes': []})
-            pt['nodes'].append(entry['node'])
-            _possible_types[name] = pt
-
-    possible_types = []
-    for t in all_types:
-        entry = _possible_types.get(t.type_info.name)
-        if entry:
-            possible_types.append(entry)
-
-    return possible_parents, possible_types
 
 def disambiguate_name(name):
     parts = name.split(u'-')
@@ -309,7 +274,7 @@ def disambiguate_name(name):
         except ValueError:
             parts.append(u'1')
         else:
-            parts[-1] = unicode(index+1)
+            parts[-1] = unicode(index + 1)
     else:
         parts.append(u'1')
     return u'-'.join(parts)
@@ -325,69 +290,138 @@ def ensure_view_selector(func):
     wrapper.__doc__ = func.__doc__
     return wrapper
 
-class FormController(object):
-    add = None
-    post_key = 'save'
-    edit_success_msg = u"Your changes have been saved."
-    add_success_msg = u"Successfully added item."
-    error_msg = (u"There was a problem with your submission.\n"
-                 u"Errors have been highlighted.")
-    success_path = '@@edit'
+class NavigationNodeWrapper(object):
+    def __init__(self, node, request, item_mapping, item_to_children):
+        self._node = node
+        self._request = request
+        self._item_mapping = item_mapping
+        self._item_to_children = item_to_children
 
-    def __init__(self, form, **kwargs):
-        self.form = form
-        for key, value in kwargs.items():
-            if key in self.__class__.__dict__:
-                setattr(self, key, value)
-            else: # pragma: no coverage
-                raise TypeError("Unknown argument %r" % key)
+    @property
+    def __parent__(self):
+        if self.parent_id:
+            return self._item_mapping[self.parent_id]
 
-    def __call__(self, context, request):
-        if self.post_key in request.POST:
-            controls = request.POST.items()
-            try:
-                appstruct = self.form.validate(controls)
-            except ValidationFailure, e:
-                request.session.flash(self.error_msg, 'error')
-                return e.render()
-            else:
-                if self.add is None: # edit
-                    return self.edit_item(context, request, appstruct)
-                else: # add
-                    return self.add_item(context, request, appstruct)
-        else: # no post means less action
-            if self.add is None:
-                appstruct = self.appstruct(context)
-                return self.form.render(appstruct)
-            else:
-                return self.form.render()
+    @property
+    def children(self):
+        return [NavigationNodeWrapper(
+            child, self._request, self._item_mapping, self._item_to_children)
+                for child in self._item_to_children[self.id]
+                if has_permission('view', child, self._request)]
 
-    def appstruct(self, item):
-        return item.__dict__.copy()
+    def __getattr__(self, name):
+        return getattr(self._node, name)
 
-    def edit(self, item, **appstruct):
-        for key, value in appstruct.items():
-            setattr(item, key, value)
+def nodes_tree(request):
+    item_mapping = {}
+    item_to_children = defaultdict(lambda: [])
+    for node in DBSession.query(Content).with_polymorphic(Content):
+        item_mapping[node.id] = node
+        if has_permission('view', node, request):
+            item_to_children[node.parent_id].append(node)
 
-    def edit_item(self, context, request, appstruct):
-        self.edit(context, **appstruct)
-        request.session.flash(self.edit_success_msg, 'success')
-        try:
-            location = resource_url(context, request) + self.success_path
-        except AttributeError:
-            location = request.url
+    for children in item_to_children.values():
+        children.sort(key=lambda ch:ch.position)
+
+    return NavigationNodeWrapper(
+        item_to_children[None][0],
+        request,
+        item_mapping,
+        item_to_children,
+        )
+
+class Form(deform.Form):
+    """A Form that allows 'appstruct' to be set on the instance.
+    """
+    def render(self, appstruct=None, readonly=False):
+        if appstruct is None:
+            appstruct = getattr(self, 'appstruct', colander.null)
+        return super(Form, self).render(appstruct, readonly)
+
+class BaseFormView(FormView):
+    form_class = Form
+    buttons = (Button('save', _(u'Save')), Button('cancel', _(u'Cancel')))
+    success_message = _(u"Your changes have been saved.")
+    success_url = None
+    schema_factory = None
+    use_csrf_token = True
+    add_template_vars = ()
+
+    def __init__(self, context, request, **kwargs):
+        self.context = context
+        self.request = request
+        self.__dict__.update(kwargs)
+
+    def __call__(self):
+        if self.schema_factory is not None:
+            self.schema = self.schema_factory()
+        if self.use_csrf_token and 'csrf_token' not in self.schema:
+            self.schema.children.append(CSRFSchema()['csrf_token'])
+        result = super(BaseFormView, self).__call__()
+        if isinstance(result, dict):
+            result.update(self.more_template_vars())
+        return result
+
+    def cancel_success(self, appstruct):
+        return HTTPFound(location=self.request.url)
+
+    def more_template_vars(self):
+        result = {}
+        for name in self.add_template_vars:
+            result[name] = getattr(self, name)
+        return result
+
+class EditFormView(BaseFormView):
+    add_template_vars = ('first_heading',)
+
+    def before(self, form):
+        form.appstruct = self.context.__dict__.copy()
+
+    def save_success(self, appstruct):
+        appstruct.pop('csrf_token', None)
+        self.edit(**appstruct)
+        self.request.session.flash(self.success_message, 'success')
+        location = self.success_url or self.request.resource_url(self.context)
         return HTTPFound(location=location)
-        
-    def add_item(self, context, request, appstruct):
+
+    def edit(self, **appstruct):
+        for key, value in appstruct.items():
+            setattr(self.context, key, value)
+
+    @reify
+    def first_heading(self):
+        return _(u'Edit <em>${title}</em>',
+                 mapping=dict(title=self.request.context.title)
+                 )
+
+class AddFormView(BaseFormView):
+    success_message = _(u"Successfully added item.")
+    item_type = None
+    add_template_vars = ('first_heading',)
+
+    def save_success(self, appstruct):
+        appstruct.pop('csrf_token', None)
+        name = self.find_name(appstruct)
+        new_item = self.context[name] = self.add(**appstruct)
+        self.request.session.flash(self.success_message, 'success')
+        location = self.success_url or self.request.resource_url(new_item)
+        return HTTPFound(location=location)
+
+    def find_name(self, appstruct):
         name = appstruct.get('name')
         if name is None:
             name = title_to_name(appstruct['title'])
-            while name in context.keys():
+            while name in self.context.keys():
                 name = disambiguate_name(name)
-        item = context[name] = self.add(**appstruct)
-        request.session.flash(self.add_success_msg, 'success')
-        location = resource_url(item, request) + self.success_path
-        return HTTPFound(location=location)
+        return name
 
-def is_root(context, request):
-    return context is TemplateAPI(context, request).root
+    @reify
+    def first_heading(self):
+        context_title = getattr(self.request.context, 'title', None)
+        type_title = self.item_type or self.add.type_info.title
+        if context_title:
+            return _(
+                u'Add ${type} to <em>${title}</em>',
+                mapping=dict(type=translate(type_title), title=context_title))
+        else:
+            return _(u'Add ${type}', mapping=dict(type=translate(type_title)))
